# -*- coding: utf-8 -*-
"""
Johannes Maierhofer 03.08.2019
"""

from netgen.csg import *
from ngsolve import *

# %%

geo = CSGeometry()
left  = Plane (Pnt(0,0,0), Vec(0,0,-1) ).bc('left')
right = Plane (Pnt(0,0,100), Vec( 0,0,1) ).bc('right')
front = Plane (Pnt(0,15,0), Vec(0,1,0) ).bc('front')
back  = Plane (Pnt(0,-15,0), Vec(0,-1,0) ).bc('back')
bot   = Plane (Pnt(5,0,0), Vec(1,0,0) ).bc('bot')
top   = Plane (Pnt(-5,0,0), Vec(-1,0, 0) ).bc('top')

bar = left * right * front * back * bot * top

cylinder = Cylinder (Pnt(-15,0,50),Pnt(15,0,50),5)


geo.Add ((bar-cylinder).mat("copper"))

mesh = Mesh(geo.GenerateMesh(maxh=3))
Draw(mesh)

# %% --------------- CURRENT FLOW ------------------------------------------------
# Material definition:
sigma = 58.1e6 # A/V/m

# Funktionsraum
fes = H1(mesh, order=1,definedon=mesh.Materials('copper'),dirichlet='left')
print ("ndof = ", fes.ndof)
u,v = fes.TnT()
gfu_pot = GridFunction(fes)  # solution

a = BilinearForm(fes, symmetric=True)
a += SymbolicBFI(sigma*grad(u)*grad(v))
a.Assemble()

scurrent = CoefficientFunction((0,0,1))
normal = CoefficientFunction((0,0,-1))

f = LinearForm(fes)
#f += SymbolicLFI(0)
f += SymbolicLFI(scurrent*normal*v, definedon=mesh.Boundaries('left'))
f += SymbolicLFI(scurrent*normal*v, definedon=mesh.Boundaries('right'))
f.Assemble()


c = Preconditioner(a,"local")
c.Update()
BVP(bf=a,lf=f,gf=gfu_pot,pre=c).Do()


cur_dens = -sigma*grad(gfu_pot)
#cur_dens = -sigma*gfu_cur.Deriv()

# %% Paraview Output

conductor_domain = CoefficientFunction([1 if mat== "copper" else 0 for mat in mesh.GetMaterials()])

# VTKOutput object
vtk = VTKOutput(ma=mesh,
               coefs=[conductor_domain, gfu_pot, cur_dens],
               names = ["conductor-domain","solution-potential","current-flow"],
                filename="ConductingBarCresult",
                subdivision=0)
vtk.Do()